local ADDON_NAME, Core = ...

--<
local match, gsub, format = string.match, string.gsub, string.format
local UI = Core.UI
local utils = Core.utils
local Sessions = Core.Sessions
-->

local frameCache = {}

local typingStatus = CreateFrame("Frame", nil, UI.body)
typingStatus:Hide()
typingStatus:SetSize(220, 16)
typingStatus:SetPoint("BOTTOMLEFT", 3, 5)
typingStatus:SetFrameStrata("HIGH")
typingStatus.BG = typingStatus:CreateTexture(nil)
typingStatus.BG:SetTexture([[Interface\Cooldown\LoC-ShadowBG]])
typingStatus.BG:SetAllPoints()
typingStatus.BG:SetTexCoord(0.3, 1, 0, 1)
typingStatus.text = typingStatus:CreateFontString(nil)
typingStatus.text:SetFontObject("NumberFont_Shadow_Med")
typingStatus.text:SetPoint("LEFT")
typingStatus.text:SetJustifyH("LEFT")


local function onHyperLinkEnter (self, link, text)
	if not Sessions.selected.isDM and link:find("^player") then
		link = link:gsub("player:", "")
		GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
		GameTooltip_SetTitle(GameTooltip, link)
		GameTooltip:Show()
	end
end


local function onHyperLinkClick (self, link, text, button)
	if link:match("^https?://") then
		StaticPopup_Show("SESSIONS_COPYURL", nil, nil, link)
		return
	end

	ChatFrame_OnHyperlinkShow(self, link, text, button)
end


local function onMouseWheel (self, delta)
	if delta == -1 and not self:AtBottom() then
		self:ScrollDown()

	elseif delta == 1 then
		self:ScrollUp()
	end

	self.slider:SetValue(-self:GetScrollOffset())
end


local function onMouseDown (self, btn)
	if Sessions.selected.isDM or btn == "RightButton" then
		return
	end
	
	local _, lineIndex = self:FindCharacterAndLineIndexAtCoordinate(self:GetScaledCursorPosition())
	local line = self.visibleLines[lineIndex]
	if not line or not match(line:GetText(), ": Click To Show]") then
		return
	end
	local pending = select(6, self:UnpackageEntry(line.messageInfo))
	line.messageInfo.message = pending
	line:SetText(pending)
end


local function onLayoutRefreshed (self)
	local lineOne = self.visibleLines[1]
	if lineOne ~= nil then
		lineOne:ClearAllPoints()
		lineOne:SetPoint("BOTTOMLEFT", 0, 2)
	end
end


local function Update (self, force)
	if not self:IsShown() then return end
	
	self.slider:SetSize(10, self:GetHeight())
	self.slider:SetMinMaxValues(-self:GetMaxScrollRange(), 0)
	
	if force then
		self.slider:SetValue(self:GetMaxScrollRange())
	end
end


local function ClearTypingStatus (self)
	typingStatus.text:SetText("")
	typingStatus:Hide()
end

local function SetTypingStatus (self, name)
	typingStatus.text:SetText([[|TInterface\CHATFRAME\UI-ChatWhisperIcon:16:16|t]].. name.. " is typing...")
	typingStatus:Show()
end


local function Close (self)
	self:Clear()
	frameCache[#frameCache+1] = self
end



function Core.CreatePage (session)
	if #frameCache > 0 then
		local page = table.remove(frameCache)
		page:SetFading(not session.isDM)
		return page
	end

	local f = CreateFrame("ScrollingMessageFrame", nil, UI.body)
	f:SetPoint("BOTTOMLEFT", 5, 6)
	f:SetPoint("TOPRIGHT", -10, -5)
	f:SetFontObject(Core.ChatFont)
	f:SetJustifyH("LEFT")
	f:SetFading(not session.isDM)
	f:SetIndentedWordWrap(true)
	f:SetInsertMode(2)
	f:EnableMouseWheel(1)
	f:SetHyperlinksEnabled(1)
	f:EnableMouse(true)
	f:SetTimeVisible(120)
	
	if not UI.body.slider then 
		UI.body.slider = Core.CreateSlider(UI.body)
		UI.body.slider:ClearAllPoints()
		UI.body.slider:SetPoint("TOPRIGHT", -1, -5) ; UI.body.slider:SetPoint("BOTTOMRIGHT", -1, 5)
	end
	
	f.slider = UI.body.slider
	f.Update = Update
	f.Close = Close
	f.SetTypingStatus = SetTypingStatus
	f.ClearTypingStatus = ClearTypingStatus
	f:SetScript("OnMouseWheel", onMouseWheel)
	f:SetScript("OnMouseDown", onMouseDown)	
	f:HookScript("OnHyperlinkClick", onHyperLinkClick)
	f:HookScript("OnHyperlinkEnter", onHyperLinkEnter)
	f:HookScript("OnHyperlinkLeave", GameTooltip_Hide)	
	hooksecurefunc(f, "RefreshLayout", onLayoutRefreshed)
	
	return f
	
end